@@ -29,6 +29,7 @@ use solana_program::entrypoint::ProgramResult;
 use solana_program::instruction::AccountMeta;
 use solana_program::program_error::ProgramError;
 use solana_program::pubkey::Pubkey;
+use std::collections::BTreeMap;
 use std::io::Write;
 
 mod account_meta;
@@ -37,7 +38,7 @@ mod bpf_upgradeable_state;
 mod common;
 pub mod context;
 mod ctor;
-mod error;
+pub mod error;
 #[doc(hidden)]
 pub mod idl;
 mod system_program;
@@ -80,6 +81,7 @@ pub trait Accounts<'info>: ToAccountMetas + ToAccountInfos<'info> + Sized {
         program_id: &Pubkey,
         accounts: &mut &[AccountInfo<'info>],
         ix_data: &[u8],
+        bumps: &mut BTreeMap<String, u8>,
     ) -> Result<Self, ProgramError>;
 }
 
@@ -87,7 +89,10 @@ pub trait Accounts<'info>: ToAccountMetas + ToAccountInfos<'info> + Sized {
 /// should be done here.
 pub trait AccountsExit<'info>: ToAccountMetas + ToAccountInfos<'info> {
     /// `program_id` is the currently executing program.
-    fn exit(&self, program_id: &Pubkey) -> ProgramResult;
+    fn exit(&self, _program_id: &Pubkey) -> ProgramResult {
+        // no-op
+        Ok(())
+    }
 }
 
 /// The close procedure to initiate garabage collection of an account, allowing
@@ -120,6 +125,15 @@ pub trait ToAccountInfo<'info> {
     fn to_account_info(&self) -> AccountInfo<'info>;
 }
 
+impl<'info, T> ToAccountInfo<'info> for T
+where
+    T: AsRef<AccountInfo<'info>>,
+{
+    fn to_account_info(&self) -> AccountInfo<'info> {
+        self.as_ref().clone()
+    }
+}
+
 /// A data structure that can be serialized and stored into account storage,
 /// i.e. an
 /// [`AccountInfo`](../solana_program/account_info/struct.AccountInfo.html#structfield.data)'s
@@ -209,9 +223,12 @@ pub trait Key {
     fn key(&self) -> Pubkey;
 }
 
-impl Key for Pubkey {
+impl<'info, T> Key for T
+where
+    T: AsRef<AccountInfo<'info>>,
+{
     fn key(&self) -> Pubkey {
-        *self
+        *self.as_ref().key
     }
 }
 
@@ -220,7 +237,7 @@ impl Key for Pubkey {
 pub mod prelude {
     pub use super::{
         access_control, account, accounts::account::Account,
-        accounts::loader_account::AccountLoader, accounts::program::Program,
+        accounts::account_loader::AccountLoader, accounts::program::Program,
         accounts::signer::Signer, accounts::system_account::SystemAccount,
         accounts::sysvar::Sysvar, accounts::unchecked_account::UncheckedAccount, constant,
         context::Context, context::CpiContext, declare_id, emit, error, event, interface, program,
@@ -252,6 +269,7 @@ pub mod prelude {
 }
 
 /// Internal module used by macros and unstable apis.
+#[doc(hidden)]
 pub mod __private {
     // Modules with useful information for users
     // don't use #[doc(hidden)] on these
@@ -263,24 +281,22 @@ pub mod __private {
     /// The starting point for user defined error codes.
     pub const ERROR_CODE_OFFSET: u32 = 6000;
 
-    #[doc(hidden)]
     pub use crate::ctor::Ctor;
-    #[doc(hidden)]
+
     pub use crate::error::Error;
-    #[doc(hidden)]
+
     pub use anchor_attribute_account::ZeroCopyAccessor;
-    #[doc(hidden)]
+
     pub use anchor_attribute_event::EventIndex;
-    #[doc(hidden)]
+
     pub use base64;
-    #[doc(hidden)]
+
     pub use bytemuck;
-    #[doc(hidden)]
+
     use solana_program::program_error::ProgramError;
-    #[doc(hidden)]
+
     use solana_program::pubkey::Pubkey;
-    #[doc(hidden)]
-    #[doc(hidden)]
+
     pub mod state {
         pub use crate::accounts::state::*;
     }
